 /* Name: main.c
 * Project: hid-mouse, a very simple HID example
 * Author: Christian Starkjohann
 * Creation Date: 2008-04-07
 * Tabsize: 4
 * Copyright: (c) 2008 by OBJECTIVE DEVELOPMENT Software GmbH
 * License: GNU GPL v2 (see License.txt), GNU GPL v3 or proprietary (CommercialLicense.txt)
 * This Revision: $Id: main.c 692 2008-11-07 15:07:40Z cs $
 */

/*
This example should run on most AVRs with only little changes. No special
hardware resources except INT0 are used. You may have to change usbconfig.h for
different I/O pins for USB. Please note that USB D+ must be the INT0 pin, or
at least be connected to INT0 as well.

We use VID/PID 0x046D/0xC00E which is taken from a Logitech mouse. Don't
publish any hardware using these IDs! This is for demonstration only!
*/

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */

#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"
#include "oddebug.h"        /* This is also an example for using debug macros */

#include <stdio.h>
#include <avr\io.h>
#include <avr\interrupt.h>

typedef struct pozitie
{
	char x, y, z;
} pozitie;

pozitie v = {0, 0, 0};
static int x0, y0, z0;

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

PROGMEM char usbHidReportDescriptor[52] = { /* USB report descriptor, size must match usbconfig.h */
    0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
    0x09, 0x02,                    // USAGE (Mouse)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x09, 0x01,                    //   USAGE (Pointer)
    0xA1, 0x00,                    //   COLLECTION (Physical)
    0x05, 0x09,                    //     USAGE_PAGE (Button)
    0x19, 0x01,                    //     USAGE_MINIMUM
    0x29, 0x03,                    //     USAGE_MAXIMUM
    0x15, 0x00,                    //     LOGICAL_MINIMUM (0)
    0x25, 0x01,                    //     LOGICAL_MAXIMUM (1)
    0x95, 0x03,                    //     REPORT_COUNT (3)
    0x75, 0x01,                    //     REPORT_SIZE (1)
    0x81, 0x02,                    //     INPUT (Data,Var,Abs)
    0x95, 0x01,                    //     REPORT_COUNT (1)
    0x75, 0x05,                    //     REPORT_SIZE (5)
    0x81, 0x03,                    //     INPUT (Const,Var,Abs)
    0x05, 0x01,                    //     USAGE_PAGE (Generic Desktop)
    0x09, 0x30,                    //     USAGE (X)
    0x09, 0x31,                    //     USAGE (Y)
    0x09, 0x38,                    //     USAGE (Wheel)
    0x15, 0x81,                    //     LOGICAL_MINIMUM (-127)
    0x25, 0x7F,                    //     LOGICAL_MAXIMUM (127)
    0x75, 0x08,                    //     REPORT_SIZE (8)
    0x95, 0x03,                    //     REPORT_COUNT (3)
    0x81, 0x06,                    //     INPUT (Data,Var,Rel)
    0xC0,                          //   END_COLLECTION
    0xC0,                          // END COLLECTION
};
/* This is the same report descriptor as seen in a Logitech mouse. The data
 * described by this descriptor consists of 4 bytes:
 *      .  .  .  .  . B2 B1 B0 .... one byte with mouse button states
 *     X7 X6 X5 X4 X3 X2 X1 X0 .... 8 bit signed relative coordinate x
 *     Y7 Y6 Y5 Y4 Y3 Y2 Y1 Y0 .... 8 bit signed relative coordinate y
 *     W7 W6 W5 W4 W3 W2 W1 W0 .... 8 bit signed relative coordinate wheel
 */
typedef struct{
    uchar   buttonMask;
    char    dx;
    char    dy;
    char    dWheel;
}report_t;

static report_t reportBuffer;
static uchar    idleRate;   /* repeat rate for keyboards, never used for mice */


usbMsgLen_t usbFunctionSetup(uchar data[8])
{
usbRequest_t    *rq = (void *)data;

    /* The following requests are never used. But since they are required by
     * the specification, we implement them in this example.
     */
    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){    /* class request type */
        DBG1(0x50, &rq->bRequest, 1);   /* debug output: print our request */
        if(rq->bRequest == USBRQ_HID_GET_REPORT){  /* wValue: ReportType (highbyte), ReportID (lowbyte) */
            /* we only have one report type, so don't look at wValue */
            usbMsgPtr = (void *)&reportBuffer;
            return sizeof(reportBuffer);
        }else if(rq->bRequest == USBRQ_HID_GET_IDLE){
            usbMsgPtr = &idleRate;
            return 1;
        }else if(rq->bRequest == USBRQ_HID_SET_IDLE){
            idleRate = rq->wValue.bytes[1];
        }
    }else{
        /* no vendor specific requests implemented */
    }
    return 0;   /* default for not implemented requests: return no data back to host */
}

/* ------------------------------------------------------------------------- */

int uart_putchar(char c, FILE *unused);
int uart_getchar(FILE* f);
FILE mystdout = FDEV_SETUP_STREAM(uart_putchar, NULL, _FDEV_SETUP_WRITE);
FILE mystdin =  FDEV_SETUP_STREAM(NULL, uart_getchar, _FDEV_SETUP_READ);


char val;
uint8_t channel = 0;

char ADC_get(uint8_t channel)
{	
	//setati canalul de pe care vreti sa faceti citirea
	ADMUX &= 0xE0;
	ADMUX |= channel;
	ADCSRA |= _BV(ADEN)|_BV(ADSC);
	//asteptati sa se termine o conversie
	while(!(ADCSRA & (1<<ADIF)));
	//returnati rezultatul conversiei
	return ADCH;
	
}

#define CX PA7
#define CY PA6
#define CZ PA5

int abs (int v)
{
	if (v >= 0) return v;
	else return -v;
}
#define sgn(val) (val > 0 ? 1 : -1)

char citit;

void newPoz ()
{
	int x=0, y=0, z=0;
	x = ADC_get (CX);
	y = ADC_get (CY);
	z = ADC_get (CZ);
	
	x -= x0;
	y -= y0;
	z -= z0;
	
	/*int sgn = x < 0 ? -1 : 1;
	
	
	if (abs(x) > 35) x = sgn * 40;
	else if (abs(x) > 25) x = sgn * 30;
	else if (abs(x) > 15) x = sgn * 20;
	else if (abs(x) > 5) x = sgn * 10;
	else x = 0;
	
	sgn = y < 0 ? -1 : 1;
	
	if (abs(y) > 35) y = sgn * 40;
	else if (abs(y) > 25) y = sgn * 30;
	else if (abs(y) > 15) y = sgn * 20;
	else if (abs(y) > 5) y = sgn * 10;
	else y = 0;
	
	sgn = y < 0 ? -1 : 1;
	
	if (abs(z) > 35) z = sgn * 40;
	else if (abs(z) > 25) z = sgn * 30;
	else if (abs(z) > 15) z = sgn * 20;
	else if (abs(z) > 5) z = sgn * 10;
	else z = 0;*/
	
	//printf ("\n%d %d ", x, xv);
	
	if (abs (x + v.x) < 127) v.x += x;
	
	if (abs (y + v.y) < 127) v.y += y;	
	
	if (abs (z + v.z) < 127) v.z += z;	
}

void ADC_init(void)
{
	// AVCC cu condesator pe AREF; PA7 ca input single ended; rezultat left-adjusted (8 biti de date)
	ADMUX = (1<<REFS0) | (1<<MUX1) | _BV(ADLAR);
	// activeaza ADC si intreruperea de conversie. Prescaler de ceas la maximum
	ADCSRA = (1<<ADEN) | (1<<ADPS2) | (1<<ADPS1) | (1<<ADPS0);

	
}

void USART_init(void)
{
	// Initializare modul USART pt comunicatie la 9600bps, 8 biti de date, la 16MHz											
	UBRRH = 0;				
	UBRRL = 103;
	UCSRB = 0x18;
	UCSRC = _BV(URSEL) | _BV(UCSZ1) | _BV(UCSZ0);
}


void Timer1_init(void)
{
// frecventa = 16kHz
	OCR1A = 320;
	TCCR1A = 0;
	TCCR1B = (1<<WGM12)|(1<<CS12)|(1<<CS10) | (1<<CS12);
	TIMSK |= _BV(OCIE1A);
}


ISR (TIMER1_COMPA_vect)
{		
	//read = 1;
	TIMSK = TIMSK & (~_BV(OCIE1A));
	sei ();
	newPoz ();
	TIMSK = TIMSK | _BV(OCIE1A);
}

void reset ()
{

	
	x0 = ADC_get (CX);
	x0 += ADC_get (CX);
	x0 += ADC_get (CX);
	x0 += ADC_get (CX);
	x0 += ADC_get (CX);
	x0 += ADC_get (CX);
	x0 += ADC_get (CX);
	x0 += ADC_get (CX);
	x0 = x0>>3;
	
	y0 = ADC_get (CY);
	y0 += ADC_get (CY);
	y0 += ADC_get (CY);
	y0 += ADC_get (CY);
	y0 += ADC_get (CY);
	y0 += ADC_get (CY);
	y0 += ADC_get (CY);
	y0 += ADC_get (CY);
	y0 = y0 >> 3;
	
	z0 = ADC_get (CZ);
	z0 += ADC_get (CZ);
	z0 += ADC_get (CZ);
	z0 += ADC_get (CZ);
	z0 += ADC_get (CZ);
	z0 += ADC_get (CZ);
	z0 += ADC_get (CZ);
	z0 += ADC_get (CZ);
	z0 = z0 >> 3;
	
	v.x = v.y = v.z = 0;
	
	//printf ("\n%d %d %d", sizeof(char), sizeof(short), sizeof(int));
}


int main(void)
{
uchar   i;

    wdt_enable(WDTO_1S);
    /* Even if you don't use the watchdog, turn it off here. On newer devices,
     * the status of the watchdog (on/off, period) is PRESERVED OVER RESET!
     */
    DBG1(0x00, 0, 0);       /* debug output: main starts */
    /* RESET status: all port bits are inputs without pull-up.
     * That's the way we need D+ and D-. Therefore we don't need any
     * additional hardware initialization.
     */
	 
	 

	ADC_init();
	
	reset ();
	 
    odDebugInit();
    usbInit();
    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
	
	uint8_t key;
	
	USART_init();
	
	stdin = &mystdin;
	stdout = &mystdout;
	DDRD &= (~_BV (PD6));
	PORTD = PORTD | (1<<PD6);
	
	Timer1_init();
	//Timer0_init ();
	
    i = 0;
    while(--i){             /* fake USB disconnect for > 250 ms */
        wdt_reset();
        _delay_ms(1);
    }
    usbDeviceConnect();
    sei();
    DBG1(0x01, 0, 0);       /* debug output: main loop starts */
    for(;;)
	{                /* main event loop */
	
	
		DBG1(0x02, 0, 0);   /* debug output: main loop iterates */
        wdt_reset();
        usbPoll();
        if(usbInterruptIsReady())
		{			
			reportBuffer.dx = abs(v.x/10) > 5 ? v.x/10 : 0;
			reportBuffer.dy = abs(v.y/10) > 5 ? v.y/10 : 0;
			reportBuffer.dWheel = abs(v.z/10) > 5 ? v.z/10 : 0;	
            DBG1(0x03, 0, 0);   /* debug output: interrupt report prepared */
            usbSetInterrupt((void *)&reportBuffer, sizeof(reportBuffer));
        }
		if (!(PIND & (1<<PD6)))
		{
			reset ();
		}
	}
    return 0;
}

// Functie ce trateaza trimiterea unui caracter pe seriala
int uart_putchar(char c, FILE *unused)      
{
	if (c == '\n')
		uart_putchar('\r', 0);
	loop_until_bit_is_set(UCSRA, UDRE);
	UDR = c;
	return 0;
}

// Functie ce trateaza primirea unui caracter pe seriala
int uart_getchar(FILE* f)
{
	char c;
    
    loop_until_bit_is_set(UCSRA, RXC);
    c = UDR;
    if(c=='\r')
		c='\n';
    uart_putchar(c,NULL); 
    return c;
}

/* ------------------------------------------------------------------------- */
